"use strict";

require("babel-polyfill");

function isPromise(val) {
  return val.then && typeof val.then === 'function';
}

function fallbackWrapper(generator) {
  var iterator = generator();

  function loopNext(val) {
    var returned = iterator.next(val);
    handleReturned(returned);
  }

  function loopError(e) {
    var returned = iterator.throw(e);
    handleReturned(returned);
  }

  function handleReturned(returned) {
    if (!returned.done) {
      if (isPromise(returned.value)) {
        returned.value.then(loopNext, loopError);
      } else {
        process.nextTick(function () {
          loopNext(returned.value);
        });
      }
    }
  }

  loopNext();
}

function fallback(fpList, Promise) {
  return new Promise(function (resolve, reject) {
    fallbackWrapper(regeneratorRuntime.mark(function g() {
      var finalError, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, fp, result;

      return regeneratorRuntime.wrap(function g$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              finalError = void 0;
              _iteratorNormalCompletion = true;
              _didIteratorError = false;
              _iteratorError = undefined;
              _context.prev = 4;
              _iterator = fpList[Symbol.iterator]();

            case 6:
              if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                _context.next = 21;
                break;
              }

              fp = _step.value;
              _context.prev = 8;
              _context.next = 11;
              return fp();

            case 11:
              result = _context.sent;
              return _context.abrupt("return", resolve(result));

            case 15:
              _context.prev = 15;
              _context.t0 = _context["catch"](8);

              finalError = _context.t0;

            case 18:
              _iteratorNormalCompletion = true;
              _context.next = 6;
              break;

            case 21:
              _context.next = 27;
              break;

            case 23:
              _context.prev = 23;
              _context.t1 = _context["catch"](4);
              _didIteratorError = true;
              _iteratorError = _context.t1;

            case 27:
              _context.prev = 27;
              _context.prev = 28;

              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }

            case 30:
              _context.prev = 30;

              if (!_didIteratorError) {
                _context.next = 33;
                break;
              }

              throw _iteratorError;

            case 33:
              return _context.finish(30);

            case 34:
              return _context.finish(27);

            case 35:
              reject(finalError);

            case 36:
            case "end":
              return _context.stop();
          }
        }
      }, g, this, [[4, 23, 27, 35], [8, 15], [28,, 30, 34]]);
    }));
  });
}

module.exports = {
  fallback: fallback,

  retry: function retry(fp, times, Promise) {
    var gen = regeneratorRuntime.mark(function gen() {
      return regeneratorRuntime.wrap(function gen$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (!(times >= 0)) {
                _context2.next = 9;
                break;
              }

              _context2.next = 3;
              return fp;

            case 3:
              times -= 1;

              if (!(times === 0)) {
                _context2.next = 6;
                break;
              }

              return _context2.abrupt("return");

            case 6:
              if (times < 0) {
                times = 0;
              }
              _context2.next = 0;
              break;

            case 9:
            case "end":
              return _context2.stop();
          }
        }
      }, gen, this);
    });

    return fallback(gen(), Promise);
  }
};